@@ -1,4 +1,4 @@
-context("URL Operations")
+# URL Operations Tests
 
 test_that("scan_url validates input correctly", {
   expect_error(scan_url(), "Must specify a valid URL")
@@ -32,4 +32,4 @@ test_that("URL operations work with mocked responses", {
   
   expect_true(exists("scan_url"))
   expect_true(exists("url_report"))
-})
\ No newline at end of file
+})
